@@ -5,9 +5,10 @@ use Mojo::Util qw(class_to_path decamelize camelize);
 use Getopt::Long qw(GetOptionsFromArray :config auto_abbrev
   gnu_compat no_ignore_case);
 File::Spec::Functions->import(qw(catfile catdir));
+use Mojo::File 'path';
 
 our $AUTHORITY = 'cpan:BEROV';
-our $VERSION   = '0.10';
+our $VERSION   = '0.11';
 
 has args => sub { {} };
 has description => sub {
@@ -16,10 +17,11 @@ has description => sub {
     return $_[0];
   }
   return $_[0]->{description} if $_[0]->{description};
-  state $bytes   = Mojo::File->new(__FILE__)->slurp();
+  state $bytes   = path(__FILE__)->slurp();
   state $package = __PACKAGE__;
-  return $_[0]->{description} = ($bytes =~ /$package\s+-\s+(.+)\n/ && $1);
+  return $_[0]->{description} = ($bytes =~ /$package\s+-\s+(.+)\n/)[0];
 };
+
 has usage => sub { shift->extract_usage };
 has _templates_path => '';
 has '_db_helper';
@@ -78,18 +80,21 @@ has routes => sub {
 
 my $_init = sub ($self, @options) {
   return $self if $self->{_initialised};
+
+  # Make sure the "tables" argument exists as an empty array
   my $args = $self->args({tables => []})->args;
 
   GetOptionsFromArray(
     \@options,
-    'C|controller_namespace=s' => \$args->{controller_namespace},
+    'H|home_dir=s'             => \$args->{home_dir},
     'L|lib=s'                  => \$args->{lib},
+    'A|api_dir'                => \$args->{api_dir},
+    'C|controller_namespace=s' => \$args->{controller_namespace},
     'M|model_namespace=s'      => \$args->{model_namespace},
 
     # TODO: 'O|overwrite'              => \$args->{overwrite},
     'T|templates_root=s' => \$args->{templates_root},
     't|tables=s@'        => \$args->{tables},
-    'H|home_dir=s'       => \$args->{home_dir},
                      );
 
   @{$args->{tables}} = split(/\s*?\,\s*?/, join(',', @{$args->{tables}}));
@@ -100,6 +105,7 @@ my $_init = sub ($self, @options) {
   $args->{model_namespace}      //= ref($app) . '::Model';
   $args->{home_dir}             //= $app->home;
   $args->{lib}                  //= catdir($args->{home_dir}, 'lib');
+  $args->{api_dir}              //= catdir($args->{home_dir}, 'api');
   $args->{templates_root}       //= $app->renderer->paths->[0];
 
   # Find templates.
@@ -180,6 +186,7 @@ sub run ($self, %options) {
     $tmpl_file = $self->_template_path('c_class.ep');
     $self->render_template_to_file($tmpl_file, $c_file, $template_args);
 
+
     # Templates
     my $template_dir  = decamelize($class_name);
     my $template_root = $args->{templates_root};
@@ -203,6 +210,9 @@ sub run ($self, %options) {
       .= Mojo::Template->new->render_file($tmpl_file, $template_args);
   }    # end foreach tables
 
+  #OpenAPI
+  $self->generate_openapi();
+
   # Routes
   my $template_args
     = {%$args, helpers => $wrapper_helpers, routes => $self->routes};
@@ -297,13 +307,144 @@ sub generate_validation ($self, $table) {
   return $fields;
 }
 
+sub generate_openapi ($self) {
+  my $tmpl_args      = {%{$self->args}};
+  my $api_dir        = $tmpl_args->{api_dir};
+  my $api_tmpl_file  = $self->_template_path('api.json.ep');
+  my $api_file       = catfile($api_dir, 'api.json');
+  my $defs_tmpl_file = $self->_template_path('definitions.json.ep');
+  my $defs_file      = catfile($api_dir, 'definitions.json');
+  $tmpl_args->{api_title}  = ref($self->app);
+  $tmpl_args->{api_paths}  = {};
+  $tmpl_args->{api_params} = {};
+  my $api_defs = {};
+  $tmpl_args->{api_definitions} = $api_defs;
+
+  for my $t (@{$tmpl_args->{tables}}) {
+
+    # Generate descriptions for table objects.
+    my $class_name = $tmpl_args->{class_name} = camelize($t);
+    my $object_name = $class_name . 'Item';
+    $api_defs->{$class_name}{items}{'$ref'} = "/resources/$object_name";
+    $api_defs->{$class_name}{type} = 'array';
+    $api_defs->{$object_name}{description}
+      = "An object, representing one item of $class_name.";
+
+    # Generate definition and parameter description for each column.
+    $self->generate_columns_api($t, $api_defs->{$object_name}, $tmpl_args);
+  }
+
+  # $self->app->log->debug($self->app->dumper($api_defs));
+  $self->render_template_to_file($defs_tmpl_file, $defs_file, $tmpl_args);
+  $self->render_template_to_file($api_tmpl_file,  $api_file,  $tmpl_args);
+
+ # Prettify generated JSON.
+ # With this step we also make sure the generated JSON is syntactically correct.
+  my $ugly = path($defs_file)->slurp();
+  path($defs_file)
+    ->spurt(JSON::PP->new->utf8->pretty->encode(JSON::PP::decode_json($ugly)));
+
+  $ugly = path($api_file)->slurp();
+  path($api_file)
+    ->spurt(JSON::PP->new->utf8->pretty->encode(JSON::PP::decode_json($ugly)));
+
+  return;
+}
+
+sub generate_columns_api ($self, $t, $object_api_def, $tmpl_args) {
+  $object_api_def->{properties} = {};
+  $object_api_def->{required}   = [];
+  for my $col (@{$self->_column_info($t)}) {
+    my $name       = $col->{COLUMN_NAME};
+    my $size       = $col->{COLUMN_SIZE} || 0;
+    my $type       = $col->{TYPE_NAME};
+    my $param_name = camelize($name) . "Of$tmpl_args->{class_name}";
+    $tmpl_args->{api_params}{$param_name} = {name => $name};
+
+    unless ($col->{NULLABLE}) {
+      $tmpl_args->{api_params}{$param_name}{required} = Mojo::JSON->true;
+      push @{$object_api_def->{required}}, $name;
+    }
+
+    if ($type =~ /char|text|clob/i) {
+      $object_api_def->{properties}{$name}
+        = {($size ? (maxLength => $size) : ()), type => 'string'};
+      $tmpl_args->{api_params}{$param_name}{maxLength} = $size if $size;
+      $tmpl_args->{api_params}{$param_name}{type} = 'string';
+    }
+    elsif ($type =~ /INT/i) {
+      $object_api_def->{properties}{$name}
+        = {($size ? (maxLength => $size) : ()), type => 'integer'};
+      $tmpl_args->{api_params}{$param_name}{maxLength} = $size if $size;
+      $tmpl_args->{api_params}{$param_name}{type} = 'integer';
+    }
+    elsif ($type =~ /FLOAT|DOUBLE|DECIMAL|NUMBER/i) {
+      my $scale     = $col->{DECIMAL_DIGITS} || 0;
+      my $precision = $size - $scale;
+      my $pattern   = qr/^-?\d{1,$precision}(?:\.\d{0,$scale})?$/x;
+      $object_api_def->{properties}{$name} = {
+                                            ($size ? (maxLength => $size) : ()),
+                                            type    => 'number',
+                                            pattern => $pattern
+      };
+      $tmpl_args->{api_params}{$param_name}{maxLength} = $size if $size;
+      $tmpl_args->{api_params}{$param_name}{type} = 'number';
+    }
+
+    #/$t/id
+    if ($name eq 'id') {
+      my $id_in_path_param = {
+                              %{$tmpl_args->{api_params}{$param_name} || {}},
+                              in       => 'path',
+                              required => Mojo::JSON->true
+                             };
+
+      # GET and DELETE
+      push @{$tmpl_args->{show_params}}, $id_in_path_param;
+
+      # PUT
+      push @{$tmpl_args->{update_params}}, $id_in_path_param;
+
+      next;
+    }
+
+    # POST
+    push @{$tmpl_args->{store_params}},
+      {'$ref' => "definitions.json#/api_params/$param_name"};
+
+    # PUT
+    push @{$tmpl_args->{update_params}},
+      {'$ref' => "definitions.json#/api_params/$param_name"};
+
+  }    #end for my $col (@{$self->_column_info($t)})
+
+  $tmpl_args->{t} = lc $t;
+
+  state $path_tmpl_file = $self->_template_path('path.json.ep');
+  my $path_file = catfile($tmpl_args->{api_dir}, "$t.json");
+  $self->render_template_to_file($path_tmpl_file, $path_file, $tmpl_args);
+  $tmpl_args->{api_paths}{"/$t"}      = {'$ref' => "$t.json#/~1$t"};
+  $tmpl_args->{api_paths}{"/$t/{id}"} = {'$ref' => "$t.json#/~1$t~1{id}"};
+
+ # prettify generated JSON.
+ # With this step we also make sure the generated JSON is syntactically correct.
+  my $ugly = path($path_file)->slurp();
+  path($path_file)
+    ->spurt(JSON::PP->new->utf8->pretty->encode(JSON::PP::decode_json($ugly)));
+
+  # Cleanup for the next table
+  delete $tmpl_args->{$_} for (qw(t store_params update_params show_params));
+  return;
+}
+
 1;
 
+
 =encoding utf8
 
 =head1 NAME
 
-Mojolicious::Command::generate::resources - Generate M, V & C from database tables
+Mojolicious::Command::generate::resources - Generate MVC & OpenAPI RESTful API files from database tables
 
 =head1 SYNOPSIS
 
@@ -318,14 +459,16 @@ This command uses L<feature/signatures>, therefore Perl 5.20 is required.
 
 =head1 DESCRIPTION
 
-L<Mojolicious::Command::generate::resources> generates directory structure for
-a fully functional
-L<MVC|Mojolicious::Guides::Growing/"Model View Controller">
-L<set of files|Mojolicious::Guides::Growing/"REpresentational State Transfer">,
-and L<routes|Mojolicious::Guides::Routing>
-based on existing tables in your application's database. 
+An early release...
 
-This tool's purpose is to promote
+L<Mojolicious::Command::generate::resources> generates directory structure for
+a fully functional L<MVC|Mojolicious::Guides::Growing/"Model View Controller">
+L<set of files|Mojolicious::Guides::Growing/"REpresentational State Transfer">,
+L<routes|Mojolicious::Guides::Routing> and RESTful API specification in
+L<OpenAPI|https://github.com/OAI/OpenAPI-Specification> format based on
+existing tables in your application's database. 
+
+The purpose of this tool is to promote
 L<RAD|http://en.wikipedia.org/wiki/Rapid_application_development> by generating
 the boilerplate code for model (M), templates (V) and controller (C) and help
 programmers to quickly create well structured, fully functional applications.
@@ -355,6 +498,28 @@ accept. All of them, beside C<--tables>, are guessed from your application and
 usually do not need to be specified.
 
 
+=head2 H|home_dir=s
+
+Optional. Defaults to C<app-E<gt>home> (which is MyApp home directory). Used to
+set the root directory to which the files will be dumped. If you set this
+option, respectively the C<lib> and C<api> folders will be created under the
+new C<home_dir>. If you want them elsewhere, set these options explicitly.
+
+=head2 L|lib=s
+
+Optional. Defaults to C<app-E<gt>home/lib> (relative to the C<--home_dir>
+directory). If you installed L<MyApp> in some custom path and you wish to
+generate your controllers into e.g. C<site_lib>, set this option.
+
+=head2 api_dir
+
+Optional. Directory where
+the L<OpenAPI|https://github.com/OAI/OpenAPI-Specification> C<json> files will
+be generated. Defaults to C<app-E<gt>home/api> (relative to the C<--home_dir>
+directory). If you installed L<MyApp> in some custom path and you wish to
+generate your C<OpenApi> files into for example C<site_lib/MyApp/etc/api>, set
+this option explicitly.
+
 =head2 C|controller_namespace=s
 
 Optional. The namespace for the controller classes to be generated. Defaults to
@@ -373,17 +538,6 @@ configuration file. Here is an example.
   # See /perldoc/Mojolicious#routes
   app->routes->namespaces(['MyApp::C']);
 
-=head2 H|home_dir=s
-
-Optional. Defaults to C<app-E<gt>home> (which is MyApp home directory). Used to
-set the root directory to which the files will be dumped.
-
-=head2 L|lib=s
-
-Optional. Defaults to C<app-E<gt>home/lib> (relative to the C<--home_dir>
-directory). If you installed L<MyApp> in some custom path and you wish to
-generate your controllers into e.g. C<site_lib>, set this option.
-
 =head2 M|model_namespace=s
 
 Optional. The namespace for the model classes to be generated. Defaults to
@@ -405,7 +559,6 @@ C<app-E<gt>renderer-E<gt>paths> in C<myapp.conf>.
 
 Mandatory. List of tables separated by commas for which controllers should be generated.
 
-
 =head1 SUPPORT
 
 Please report bugs, contribute and make merge requests on
@@ -418,7 +571,7 @@ L<Mojolicious::Command> and implements the following new ones.
 
 =head2 args
 
-Used for storing arguments from the commandline template.
+Used for storing arguments from the command-line.
 
   my $args = $self->args;
 
@@ -427,11 +580,12 @@ Used for storing arguments from the commandline template.
   my $description = $command->description;
   $command        = $command->description('Foo!');
 
-Short description of this command, used for the commands list.
+Short description of this command, used for the C<~$ mojo generate> commands
+list.
 
 =head2 routes
 
-  $self->routes();
+  $self->routes;
 
 Returns an ARRAY reference containing routes, prepared after
 C<$self-E<gt>args-E<gt>{tables}>. Suggested Perl code for the routes is dumped
@@ -467,12 +621,30 @@ arguments to the template. See also L<Mojolicious::Command/render_to_file>.
 
 =head2 generate_formfields
 
-Generates form-fields from columns information found in the repective table.
+Generates form-fields from columns information found in the respective table.
 The result is put into C<_form.html.ep>. The programmer can then modify the
 generated form-fields.
 
     $form_fields = $self->generate_formfields($table_name);
 
+=head2 generate_openapi
+
+Generates L<Open API|https://github.com/OAI/OpenAPI-Specification> files in
+json format.  The generated files are put in L</--api_dir>. The files are:
+C<api.json> - the file which will be loaded by C<MyApp> and refers to the
+specific path files; C<definitions.json> - this file is referred to in the
+specific path files; C<$path.json> - a file for each resource, based on the
+table name from which it is generated.
+
+=head2 generate_path_api
+
+Generates API definitions and path file for each table. Invoked in
+L</generate_openapi>. B<Paramaters:> C<$t> - the table name;
+$C<$api_defs_object> - the object API definition, based on the table name;
+C<$tmpl_args> - the arguments for the templates. C<$api_defs_object> and
+$C<tmpl_args> will be enriched with additional key-value pairs as required by
+the OpenAPI specification. Returns C<void>.
+
 =head2 generate_validation
 
 Generates code for the C<_validation> method in the respective controler.
@@ -500,14 +672,15 @@ This program is free software licensed under
 
   Artistic License 2.0
 
-The full text of the license can be found in the
-LICENSE file included with this module.
+The full text of the license can be found in the LICENSE file included with
+this module.
 
 =head1 SEE ALSO
 
 L<Mojolicious::Command::generate>,
 L<Mojolicious::Command>,
 L<Mojolicious>,
+L<Mojolicious::Plugin::OpenAPI>,
 L<Perl|https://www.perl.org/>.
 
 =cut
